/*

 gds2dot.cpp : creates .dot files for a given .gds file.

 usage: gds2dot.exe <inputfile.gds>

 Created by: Alejandro Gutierrez
 Created Date: 10/18/2007
 Last Modified Date: 10/30/2007
 University of South Florida


 Copyright (C) 2008  Alejandro Gutierrez Munoz

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <iostream>
#include <fstream>
#include <ostream>
#include <sstream>
#include <iomanip>
#include <algorithm>
#include "graph.h"

using namespace std;

void ProcessFile(const string&);
int LineCount(const string&);
void ExportGraphtoDOT(const graph&);

int main(int argc, char* argv[])
{

	string filename;

	if (argc != 2) {
		cerr << "usage: gds2dot.exe <inputfile.gds>" << endl;
	}
	else {
		filename = argv[1];
		
		ProcessFile(filename);
	}
	
	return 0;

}

void ProcessFile(const string &filename) {

	ifstream gdsFile;
	int lineno, file_lines_no, number_of_graphs;
	string line, graph_string;
	graph gTemp;

	lineno = 0;
	number_of_graphs = 0;

	gdsFile.open(filename.c_str());
	file_lines_no = LineCount(filename);

	 if (gdsFile.is_open()) {
		while (!gdsFile.eof())
		{
		  lineno++;
		  getline(gdsFile,line);

		  // start reading a graph
		  if (!line.compare(0,5,"BEGIN")) {

				number_of_graphs++;

				// clear current string
				graph_string.clear(); 
				graph_string.append(line);
				graph_string.append("\n");
				
				do {
					lineno++;
					getline(gdsFile,line);
					graph_string.append(line);
					graph_string.append("\n");

				} while (line.compare(0,3,"END"));
				
		  

			  try {
				
				gTemp = graph::graph(graph_string);
				ExportGraphtoDOT(gTemp);

				// report the progress
				cout << "\b\b\b\b\b\b\b\b\b\b\b\\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\\b\b\b\b\b\b\b\b\b"  // this clears the line
					<< setiosflags(ios::fixed) << setprecision(2) << (lineno / (double) file_lines_no)*100.0 << "%\tGraph No: " << number_of_graphs << " "; 
				
			  }
			  catch (int) {
				cerr << "Error reading file line: " << lineno << endl; 
			  }

			  gTemp.~graph(); 
		  }

				 
		}
		gdsFile.close();
	 }
	 else {
		 cout << "Unable to open file"; 
	 }

	return;
}


int LineCount(const string &filename){
	
    ifstream in(filename.c_str());
	int total = count(istreambuf_iterator<char>(in),
                      istreambuf_iterator<char>(),'\n');
   
    return total;
}


void ExportGraphtoDOT(const graph &g1){

	string dotFilename;
	ofstream dotFile;
			
	dotFilename = "";
	dotFilename.append(".\\"); 
	dotFilename.append(g1.getGraphName());
	dotFilename.append(".dot");

	dotFile.open(dotFilename.c_str());
	dotFile << g1.exportDOTstring();
	dotFile.close();
	dotFilename.clear(); 
			

	return;
}
