/*
 Sdf2Gds.cpp : transforms a sdf file to a gds file.

 usage: sdf2gds <inputfile.sdf>

 Created by: Alejandro Gutierrez
 Created Date: 09/10/2007
 Last Modified Date: 11/08/2007
 University of South Florida


 Copyright (C) 2008  Alejandro Gutierrez Munoz

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

using namespace std;

int main(int argc, char* argv[])
{

	ifstream sdfFile;
	ofstream gdsFile;
	istringstream buffer;
	string line;
	string inputfile, outputfile;
	string molecule_name;
	int no_edges, no_vertices, from, to;
	string vertices;
	string edges;
	string label,from_str, to_str, value_str, screen_conclusion;
	float float_temp;
	ostringstream os;
	int line_no = 0;
	

	if (argc != 2) {
		cerr << "usage: sdf2gds <inputfile.sdf>" << endl;
	}
	else {
		
		inputfile = outputfile = argv[1];
		outputfile.append(".gds");

		sdfFile.open(inputfile.c_str());

		if (sdfFile.is_open()) {

			cout << "converting file..." << endl;

			gdsFile.open(outputfile.c_str()); 
			
			
			while (!sdfFile.eof())
			{
			
				// get first line that contains the name
				getline(sdfFile,line);
				line_no++;
				buffer.str(line);
				
				buffer >> molecule_name;
				buffer.clear();

				if (!molecule_name.empty()){
					

					// read 2 extra lines
					getline(sdfFile,line);
					line_no++;
					getline(sdfFile,line);
					line_no++;

					// read line that contains number of edges and vertices
					getline(sdfFile,line);
					line_no++;
					buffer.str(line);

					no_vertices = (int) strtod(line.substr(0,3).c_str(),NULL);
					no_edges = (int) strtod(line.substr(3,3).c_str(),NULL);

					buffer.clear();

					// once the number of vertices and edges is know, read them
					for (int v=1; v<= no_vertices; v++) {

						getline(sdfFile,line);
						line_no++;
						buffer.str(line);

						// read 3d coordinates -- not used
						buffer >> float_temp;
						buffer >> float_temp;
						buffer >> float_temp;

						// read vertex label
						buffer >> label;

						// clean the vertex label in case of leading zeros
						if (!label.compare(0,1,"0")){
							label = label.erase(0,1);
						}

						//transform the (int) v to a string
						os << v;
						
						vertices.append("v\t"); 
						vertices.append(os.str());
						vertices.append("\t"); 
						vertices.append(label);
						vertices.append("\n"); 

						os.str("");				
						buffer.clear();
					

					}

					// once the number of vertices and edges is know, read them
					for (int e=1; e<= no_edges; e++) {

						getline(sdfFile,line);
						line_no++;
						buffer.str(line);

						// read from and to and label of the edge
						from = (int) strtod(line.substr(0,3).c_str(),NULL) ;
						to = (int) strtod(line.substr(3,3).c_str(),NULL);
						float_temp = (float) strtod(line.substr(6,3).c_str(),NULL);
										
						// convert float to string
						os << from;
						from_str = os.str();
						os.str("");

						// convert float to string
						os << to;
						to_str = os.str();
						os.str("");


						// convert float to string
						os << float_temp;
						value_str = os.str();
						os.str("");


						edges.append("e\t");
						edges.append(from_str);
						edges.append("\t");
						edges.append(to_str);
						edges.append("\t");
						edges.append(value_str);
						edges.append("\n"); 

						//duplicate the edge since is undirected
						edges.append("e\t");
						edges.append(to_str);
						edges.append("\t");
						edges.append(from_str);
						edges.append("\t");
						edges.append(value_str);
						edges.append("\n"); 

						buffer.clear();

					}

					// skip rest of the file until the screen conclusion
					while (line.compare(0,41,">  <NCI_AIDS_Antiviral_Screen_Conclusion>")) {
						getline(sdfFile,line);
						line_no++;
					}
					
					// read the screen conclusion
					getline(sdfFile,line);
					line_no++;
					buffer.str(line);
					buffer >> screen_conclusion;

					buffer.clear();

					// skip rest of the end of molecule $$$
					while (line.compare(0,4,"$$$$")) {
						getline(sdfFile,line);
						line_no++;
					}

					// write results to file
					gdsFile << "BEGIN\t" << molecule_name << "\t" << screen_conclusion << endl;
					gdsFile << vertices;
					gdsFile << edges;
					gdsFile << "END\n";

					//clean everything
					vertices.clear();
					edges.clear();
					label.clear();
					from_str.clear();
					to_str.clear();
					value_str.clear();
					screen_conclusion.clear();
					molecule_name.clear();
				}

			}
			
		}

		sdfFile.close(); 
		gdsFile.close(); 

		cout << "Done!" << endl;
	}

	return 0;
}

